/* **********************************************************************
    Copyright 2003 Rensselaer Polytechnic Institute.

    All worldwide rights reserved. A license to use, copy, modify and
    distribute this software for noncommercial research purposes only is
    hereby granted, provided that this copyright notice and accompanying
    disclaimer is not modified or removed from the software.

    DISCLAIMER: The software is distributed" AS IS" without any express or
    implied warranty, including but not limited to, any implied warranties
    of merchantability or fitness for a particular purpose or any warrant)'
    of non-infringement of any current or pending patent rights. The authors
    of the software make no representations about the suitability of this
    software for any particular purpose. The entire risk as to the quality
    and performance of the software is with the user. Should the software
    prove defective, the user assumes the cost of all necessary servicing,
    repair or correction. In particular, neither Rensselaer Polytechnic
    Institute, nor the authors of the software are liable for any indirect,
    special, consequential, or incidental damages related to the software,
    to the maximum extent the law permits.
*/

package edu.rpi.cct.uwcal.calsvci;

import edu.washington.cac.calfacade.shared.AdminGroups;
import edu.washington.cac.calfacade.shared.CalFacadeException;
import edu.washington.cac.calfacade.shared.EventVO;
import edu.washington.cac.calfacade.shared.KeywordVO;
import edu.washington.cac.calfacade.shared.LocationVO;
import edu.washington.cac.calfacade.shared.MyCalendarVO;
import edu.washington.cac.calfacade.shared.SponsorVO;
import edu.washington.cac.calfacade.shared.UserAuth;
import edu.washington.cac.calfacade.shared.UserVO;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.SortedSet;
import java.util.TreeSet;

/** This is the service interface to the calendar suite. This will be
 * used by web applications and web services as well as other applications
 * which wish to integrate calendar actions into their interface.
 *
 * <p>This is a high level interface which carries out commonly used
 * calendar operations which may involve a number of interactions with the
 * underlying database implementation.
 *
 * @author Mike Douglass       douglm@rpi.edu
 */
public interface CalSvcI extends Serializable/*, Calintf*/ {
  /** (Re)initialise the object for a particular use.
   *
   * @param initObj     object used to initialise properties, for example, a
   *                    servlet.
   * @param user        String user of the application, null for guest
   * @param rights      int access rights value as defined by
   *                    edu.washington.cac.calfacade.shared.UserAuth
   * @param publicAdmin boolean true if this is a public events admin app
   * @param debug       boolean true to turn on debugging trace
   * @return Calintf    object for interactions with back-end.
   */
  public void init(Object initObj,
                   String user,
                   int rights,
                   boolean publicAdmin,
                   boolean debug) throws CalFacadeException;

  /** (Re)initialise the object for a particular use.
   *
   * @param user        String user of the application, null for guest
   * @param rights      int access rights value as defined by
   *                    edu.washington.cac.calfacade.shared.UserAuth
   * @param publicAdmin boolean true if this is a public events admin app
   * @param debug       boolean true to turn on debugging trace
   * @return Calintf    object for interactions with back-end.
   */
  public void init(String user,
                   int rights,
                   boolean publicAdmin,
                   boolean debug) throws CalFacadeException;

  /** Must be called to initialise the new object.
   *
   * @param initObj     object used to initialise properties, for example, a
   *                    servlet.
   * @param user        String user of the application
   * @param rights      int access rights value as defined by
   *                    edu.washington.cac.calfacade.shared.UserAuth
   * @param isGuest     boolean true if user is guest
   * @param publicAdmin boolean true if this is a public events admin app
   * @param debug       boolean true to turn on debugging trace
   */
  public void init(Object initObj,
                   String user,
                   int rights,
                   boolean isGuest,
                   boolean publicAdmin,
                   boolean debug) throws CalFacadeException;

  /** Returns a value object representing the current user.
   *
   * @return UserVO       representing the current user
   */
  public UserVO getUserVO() throws CalFacadeException;

  /** Get a UserAuth object which allows the application to determine what
   * special rights the user has.
   *
   * <p>Note that the returned object may require a one time initialisation
   * call to set it up correctly.
   * @see edu.washington.cac.calfacade.shared.UserAuth#initialise(String, int)
   * @see edu.washington.cac.calfacade.shared.UserAuth#initialise(String, Object)
   *
   * @return UserAuth    implementation.
   */
  public UserAuth getUserAuth() throws CalFacadeException;

  /** Get an AdminGroups object which allows the application to handle
   * administrative groups.
   *
   * @return AdminGroups    implementation.
   */
  public AdminGroups getAdminGroups() throws CalFacadeException;

  /** Refresh the users events cache
   */
  public void refreshEvents() throws CalFacadeException;

  /* * Update lastmod timestamp.
   *
   * @exception CalFacadeException If there's a database access problem
   * /
  public void touch() throws CalFacadeException;*/

  /** Return true if we believe a refresh is required. After a return of
   * true this method willreturn false until somebody updates the db.
   *
   * @return boolean    time to refresh
   */
  public boolean refreshNeeded() throws CalFacadeException;

  /** ===================================================================
   *                   Calendars and search
   *  =================================================================== */

  /** Return a list of calendar we can present to the client as a list of
   * available calendars.
   *
   * @return SortedSet   of CalendarVO
   */
  public SortedSet getCalendars() throws CalFacadeException;

  /** Set a search filter using the suppplied search string
   *
   * @param val    String search parameters
   */
  public void setSearch(String val) throws CalFacadeException;

  /** Return the current search string
   *
   * @return  String     search parameters
   */
  public String getSearch() throws CalFacadeException;

  /** Set the calendar we are interested in. This is represented by the name
   * of a calendar.
   *
   * @param  val     int id of calendar - 0 for none
   * @return boolean true for OK, false for unknown calendar
   */
  public boolean setCalendar(int val) throws CalFacadeException;

  /** Get the name of the current calendar we are displaying
   *
   * @return String    name of calendar or null for all events
   */
  public String getCalendarTitle() throws CalFacadeException;

  /** This method is called to add a calendar. No consistency checks are
   * carried out at the moment.
   *
   * <p>This is a temp fix until we have some sort of admin interface to
   * create and update calendar definitions.
   *
   * @param val  String values for db
   */
  public void addCalendarDef(String val) throws CalFacadeException;

  /** Set a filter which returns events starting after a given Timestamp.
   *
   * @param val   Timestamp object.
   */
  public void setLastmodGECalendar(Timestamp val) throws CalFacadeException;

  /** Set a filter which returns events starting after a given date of the
   * form yyyymmdd.
   *
   * @param date   String of form yyyymmdd.
   */
  public void setLastmodGEDateCalendar(String date) throws CalFacadeException;

  /** Set a filter which returns events starting after a given date and time
   * of the form yyyymmddhhmm.
   *
   * @param dateTime   String of form yyyymmddhhmm.
   */
  public void setLastmodGEDateTimeCalendar(String dateTime)
      throws CalFacadeException;

  /** ===================================================================
   *                   Subscriptions
   *  =================================================================== */

  /** Ensure that the users subscriptions match a set of calendars
   *
   * @param crs      TreeSet of Integer calendar ids
   * @return int[]   Number of calendars subscribed to, and number of
   *                 calendars unsubscribed to
   */
  public int[] fixSubscriptions(TreeSet crs) throws CalFacadeException;

  /** Determine if user is subscribed to a calendar
   *
   * @param id       int calendar id
   * @return boolean true if subscribed to calendars
   */
  public boolean getSubscribed(int id) throws CalFacadeException;

  /** ===================================================================
   *                   Keywords
   *  =================================================================== */

  /** Add a Keyword to the database. The id will be set in the parameter
   * object.
   *
   * @param val   KeywordVO object to be added
   * @return int  new sponsor id
   */
  public int addKeyword(KeywordVO val) throws CalFacadeException;

  /** Replace a keyword in the database.
   *
   * @param val   KeywordVO object to be replaced
   */
  public void replaceKeyword(KeywordVO val) throws CalFacadeException;

  /** Delete a keyword with the given id. Also remove it from the current
   * user preferences (if any).
   *
   * @param id       int id of the keyword
   * @return int     0 if it was deleted.
   *                 1 if it didn't exist
   *                 2 if in use
   */
  public int deleteKeyword(int id) throws CalFacadeException;

  /** Return alert keywords only.
   *
   * @return keywordVO[]    array of alert keywords
   */
  public KeywordVO[] getAlertKeywords() throws CalFacadeException;

  /** Search the public keywords for a matching name
   *
   * @param k          KeywordVO object to match for
   * @return KeywordVO matching object or null if no match.
   */
  public KeywordVO findKeyword(KeywordVO k) throws CalFacadeException;

  /** Ensure a keyword exists. If it does returns the keyword id.
   * If not creates the keyword then returns the new id.
   *
   * @param keyword   KeywordVO value object. If this object has the id set,
   *                  we assume the check was made previously.
   * @return int      keywordid that matches or new keywordid
   */
  public int ensureKeywordExists(KeywordVO keyword) throws CalFacadeException;

  /** ===================================================================
   *                   Public Keywords
   *  =================================================================== */

  /** Return public keywords excluding alerts.
   * This modifies th ebehaviour of the Calintf getPublicKeywords method.
   * (I don't think there's anything other than public keywords.
   *
   * @return keywordVO[]    array of keywords
   */
  public KeywordVO[] getPublicKeywords() throws CalFacadeException;

  /** Return public keywords including alerts.
   *
   * @return keywordVO[]    array of keywords
   */
  public KeywordVO[] getAllPublicKeywords() throws CalFacadeException;

  /** Return an array of the given keywords
   *
   * @param keys          Keys of the desired keywords
   * @return KeywordVO[] Alphabetically sorted array
   */
  public KeywordVO[] getPublicKeywords(int[] keys)
      throws CalFacadeException;

  /** Return given public keyword
   *
   * @param id     int id of the keyword
   * @return KeywordVO[]  array of keywords value objects
   */
  public KeywordVO getPublicKeyword(int id) throws CalFacadeException;

  /** ===================================================================
   *                   Keyword attributes
   *  =================================================================== */

  /** Set keyword alert on/off
   *
   * @param id       keyword id
   * @param val      boolean turn it on/off
   */
  public void setKeyAlert(int id, boolean val) throws CalFacadeException;

  /** Is given keyword an alert?
   *
   * @param id       keyword id
   * @return boolean true if this is an alert keyword
   */
  public boolean getKeyAlert(int id) throws CalFacadeException;

  /** ===================================================================
   *                   Locations
   *  =================================================================== */

  /** Return all locations for this user
   *
   * @return LocationVO[]  array of location value objects
   */
  public LocationVO[] getLocations() throws CalFacadeException;

  /** Add a location to the database. The id will be set in the parameter
   * object.
   *
   * @param val   LocationVO object to be added
   * @return int  new location id
   */
  public int addLocation(LocationVO val) throws CalFacadeException;

  /** Replace a location in the database.
   *
   * @param val   LocationVO object to be replaced
   */
  public void replaceLocation(LocationVO val) throws CalFacadeException;

  /** Return a location with the given id
   *
   * @param id     int id of the location
   * @return LocationVo object representing the location in question
   *                     null if it doesn't exist.
   */
  public LocationVO getLocation(int id) throws CalFacadeException;

  /** Delete a location with the given id
   *
   * @param id       int id of the location
   * @return int     0 if it was deleted.
   *                 1 if it didn't exist
   *                 2 if in use
   */
  public int deleteLocation(int id) throws CalFacadeException;

  /** ===================================================================
   *                   Public Locations
   *  =================================================================== */

  /** Return public locations - excluding reserved entries.
   *
   * @return LocationVO[]  array of location value objects
   */
  public LocationVO[] getPublicLocations() throws CalFacadeException;

  /** Return given public location
   *
   * @param id          int id of the location
   * @return LocationVo object representing the location in question
   *                     null if it doesn't exist.
   */
  public LocationVO getPublicLocation(int id) throws CalFacadeException;

  /** Search the public locations for a matching name, phone, email and link
   *
   * @param l          LocationVO object to match for
   * @return LocationVO matching object or null if no match.
   */
  public LocationVO findPublicLocation(LocationVO l) throws CalFacadeException;

  /** Ensure a location exists. If it does returns the location id.
   * If not creates the location then returns the new id.
   *
   * @param location   LocationVO value object. If this object has the id set,
   *                  we assume the check was made previously.
   * @return int      locationid that matches or new locationid
   */
  public int ensurePublicLocationExists(LocationVO location)
      throws CalFacadeException;

  /** Return an array of the given locations
   *
   * @param keys          Keys of the desired locations
   * @return LocationVO[] Alphabetically sorted array
   */
  public LocationVO[] getPublicLocations(int[] keys)
      throws CalFacadeException;

  /** Delete a public location with the given id. Also remove it from the current
   * user preferences (if any).
   *
   * @param id       int id of the location
   * @return int     0 if it was deleted.
   *                 1 if it didn't exist
   *                 2 if in use
   */
  public int deletePublicLocation(int id) throws CalFacadeException;

  /** ===================================================================
   *                   Sponsors
   *  =================================================================== */

  /** Return an array of SponsorVO objects
   *
   * @return SponsorVO[]    array of sponsors
   */
  public SponsorVO[] getSponsors() throws CalFacadeException;

  /** Add a sponsor to the database. The id will be set in the parameter
   * object.
   *
   * @param val   SponsorVO object to be added
   * @return int  new sponsor id
   */
  public int addSponsor(SponsorVO val) throws CalFacadeException;

  /** Replace a sponsor in the database.
   *
   * @param val   SponsorVO object to be replaced
   */
  public void replaceSponsor(SponsorVO val) throws CalFacadeException;

  /** Search the public sponsors for a matching name, phone, email and link
   *
   * @param s          SponsorVO object to match for
   * @return SponsorVO matching object or null if no match.
   */
  public SponsorVO findSponsor(SponsorVO s) throws CalFacadeException;

  /** Ensure a sponsor exists. If it does returns the sponsor id.
   * If not creates the sponsor then returns the new id.
   *
   * @param sponsor   SponsorVO value object. If this object has the id set,
   *                  we assume the check was made previously.
   * @return int      sponsorid that matches or new sponsorid
   */
  public int ensureSponsorExists(SponsorVO sponsor) throws CalFacadeException;

  /** Return an array of the given sponsors
   *
   * @param keys          Keys of the desired sponsors
   * @return SponsorVO[] Alphabetically sorted array
   */
  public SponsorVO[] getPublicSponsors(int[] keys)
      throws CalFacadeException;

  /** Return given public sponsor
   *
   * @param id         int id of the sponsor
   * @return SponsorVo object representing the sponsor in question
   *                     null if it doesn't exist.
   */
  public SponsorVO getPublicSponsor(int id) throws CalFacadeException;

  /** Delete a sponsor with the given id. Also remove it from the current
   * user preferences (if any).
   *
   * @param id       int id of the sponsor
   * @return int     0 if it was deleted.
   *                 1 if it didn't exist
   *                 2 if in use
   */
  public int deleteSponsor(int id) throws CalFacadeException;

  /** ===================================================================
   *                   Events
   *  =================================================================== */

  /** Return a single event for the current user
   *
   * @param   eventId   int id of the event
   * @return  EventVO   value object representing event.
   */
  public EventVO getEvent(int eventId) throws CalFacadeException;

  public static class DelEventResult {
    public boolean eventDeleted;
    public boolean locationDeleted;

    public DelEventResult(boolean eventDeleted,
                          boolean locationDeleted) {
      this.eventDeleted = eventDeleted;
      this.locationDeleted = locationDeleted;
    }
  }

  /** Delete an event. Optionally delete the event location if it is no
   * longer in use.
   *
   * @param eventid            int id of event to delete
   * @param delUnreffedLoc     delete the location if unreferenced
   * @return DelEventResult    result.
   * @exception CalFacadeException If there's a database access problem
   */
  public DelEventResult deleteEvent(int eventid,
                                    boolean delUnreffedLoc) throws CalFacadeException;

  /** Return the events for the given day as an array of value objects
   *
   * @param   date    MyCalendarVO object defining day
   * @return  EventVO[]  one days events or null for no events.
   */
  public EventVO[] getDaysEvents(MyCalendarVO date) throws CalFacadeException;

  /** Add an event to the database. The id will be set in the parameter
   * object.
   *
   * @param val   EventVO object to be added
   * @return int  new event id
   */
  public int addEvent(EventVO val) throws CalFacadeException;

  /** Replace an event with the same id in the database.
   *
   * @param val   EventVO object to be replaced
   */
  public void replaceEvent(EventVO val) throws CalFacadeException;

  /** ===================================================================
   *                   Public Events
   *  =================================================================== */

  /** Return public events for the current user which match the supplied
   * alert flag
   *
   * @param alerts      boolean true if we want alerts false for exclude alerts
   * @return EventVO[]  populated event value objects
   */
  public EventVO[] getPublicEvents(boolean alerts)
      throws CalFacadeException;

  /** Return active public events for the current user which match the
   * supplied alert flag
   *
   * @param alerts      boolean true if we want alerts false for exclude alerts
   * @return EventVO[]  populated event value objects
   */
  public EventVO[] getActivePublicEvents(boolean alerts)
      throws CalFacadeException;

  /* * Return the alerts for the given day as an array of value objects
   *
   * @param   date    MyCalendarVO object defining day
   * @return  EventVO[]  one days alerts or null for no events.
   * /
  public EventVO[] getDaysAlerts(MyCalendarVO date) throws CalFacadeException;*/

  /** Add an event and ensure location and sponsor exist. Some calendar
   * implementors choose to allow the dynamic creation of locations and
   * sponsors. For each of those, if we have an id, then the object represents
   * a preexisting database item.
   *
   * <p>Otherwise the client has provided information which will be used to
   * locate an already existing location or sponsor. Failing that we use the
   * information to create a new entry.
   *
   * @param loc        LocationVO object
   * @param sp         SponsorVO object
   * @param event      EventVO object to be added
   * @return EventVO   Update object with embedded location and sponsor.
   */
  public EventVO addPublicEvent(LocationVO loc, SponsorVO sp,
                                EventVO event) throws CalFacadeException;

  /** Update an event and ensure location and sponsor exist. Some calendar
   * implementors choose to allow the dynamic creation of locations and
   * sponsors. For each of those, if we have an id, then the object represents
   * a preexisting database item.
   *
   * <p>Otherwise the client has provided information which will be used to
   * locate an already existing location or sponsor. Failing that we use the
   * information to create a new entry.
   *
   * <p>If the location or sponsor changes, the old ones may no longer be
   * referenced by any events. If flagged this method will delete the old
   * unused entries.
   *
   * @param loc           LocationVO object
   * @param deleteOldLoc  true if we should delete old unused location
   * @param sp            SponsorVO object
   * @param deleteOldSp   true if we should delete old unused sponsor
   * @param oldKeywordId  Id of old event category. Same as newKeywordId
   *                      if unchanged.
   * @param newKeywordId  int id to use
   * @param event         EventVO object to be added
   * @return EventVO      Updated object with embedded location and sponsor.
   */
  public EventVO updatePublicEvent(LocationVO loc, boolean deleteOldLoc,
                                   SponsorVO sp, boolean deleteOldSp,
                                   int oldKeywordId,
                                   int newKeywordId,
                                   EventVO event) throws CalFacadeException;

  /** Delete an event and optionally delete the old location and/or sponsor
   * entries if now unused.
   *
   * @param deleteOldLoc  true if we should delete old unused location
   * @param deleteOldSp  true if we should delete old unused sponsor
   * @param event      EventVO object to be added
   */
  public void deletePublicEvent(boolean deleteOldLoc,
                                boolean deleteOldSp,
                                EventVO event) throws CalFacadeException;

  /** Set a public location object in an event. Does not update anything.
   *
   * @param  locId          Keys of the desired location
   * @param  event          EventVO object
   * @return LocationVO     Location object or null if none with that key
   */
  public LocationVO setEventPublicLocation(int locId, EventVO event)
      throws CalFacadeException;

  /** ===================================================================
   *                   EventRefs
   *  =================================================================== */

  /** Create an event ref for the current user.
   *
   * @param   eventId   int id of the event to add
   * @return  boolean   false for no such item.
   */
  public boolean addEventRef(int eventId) throws CalFacadeException;

  /** Delete an event ref for the current user.
   *
   * @param   eventId   int id of the event to delete
   * @return  boolean   false for no such item.
   */
  public boolean deleteEventRef(int eventId) throws CalFacadeException;
}

